class Command {
  bool _canUndo;
  bool _canExecute;
  final CommandAction actions;

  bool get canUndo => _canUndo;
  bool get canExecute => _canExecute;
  String get key => actions.key;

  Command(this.actions) {
    _canUndo = false;
    _canExecute = true;
  }

  void execute() {
    if (!_canExecute)
      throw new StateError(
        "redo cannot be performed because canExecute is false",
      );

    _canUndo = true;
    _canExecute = false;
    actions.action();
  }

  void undo() {
    if (!_canUndo)
      throw new StateError(
        "undo cannot be performed because canUndo is false",
      );

    _canUndo = false;
    _canExecute = true;
    actions.undoAction();
  }
}

abstract class CommandAction {
  String get key;
  void action();
  void undoAction();
}
