import 'package:test/test.dart';
import 'package:command/command.dart';

main() {
  test("Undo and redo work correctly", () {
    var command = new Command(new CommandActionTester());
    expect(command.canUndo, equals(false));
    expect(command.canExecute, equals(true));

    command.execute();
    expect(command.canUndo, equals(true));
    expect(command.canExecute, equals(false));

    command.undo();
    expect(command.canUndo, false);
    expect(command.canExecute, true);

    command.execute();
    expect(command.canUndo, true);
    expect(command.canExecute, false);
  });

  test("Performing redo when unable to redo throws an error", () {
    var command = new Command(new CommandActionTester())..execute();
    expect(
      () => command.execute(),
      throwsA(new isInstanceOf<StateError>()),
    );
  });

  test("Performing undo when unable to undo throws an error", () {
    var command = new Command(new CommandActionTester());
    expect(
      () => command.undo(),
      throwsA(new isInstanceOf<StateError>()),
    );
  });
}

class CommandActionTester extends CommandAction {
  void action() {}
  void undoAction() {}
  String get key => "CommandActionTester";
}
